import argparse
import os
from zipfile import ZipFile
import tarfile
import glob
import re
import sys

import shipyard_bp_utils as shipyard
from shipyard_file_manipulation import compress
from shipyard_file_manipulation.errors import (
    EXIT_CODE_UNKNOWN_ERROR,
    EXIT_CODE_FILE_NOT_FOUND,
    EXIT_CODE_NO_FILE_MATCHES,
)
from shipyard_templates import ShipyardLogger, ExitCodeException

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--compression",
        dest="compression",
        choices={"zip", "tar", "tar.bz2", "tar.gz"},
        required=True,
    )
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        choices={"exact_match", "regex_match"},
        required=True,
    )
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default="Archive",
        required=False,
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    return parser.parse_args()


def main():
    try:
        args = get_args()
        compression = args.compression
        src_file = args.source_file_name
        src_dir = shipyard.files.clean_folder_name(args.source_folder_name)
        src_path = shipyard.files.combine_folder_and_file_name(
            folder_name=src_dir, file_name=src_file
        )
        match_type = args.source_file_name_match_type
        target_dir = shipyard.files.clean_folder_name(args.destination_folder_name)
        target_file = args.destination_file_name
        target_path = shipyard.files.combine_folder_and_file_name(
            target_dir, target_file
        )

        if target_dir:
            shipyard.files.create_folder_if_dne(target_dir)

        if match_type == "regex_match":
            file_paths = shipyard.files.find_all_local_file_names(src_dir)
            matching_file_paths = shipyard.files.find_all_file_matches(
                file_paths, re.compile(src_file)
            )
            logger.debug(f"Matching files: {matching_file_paths}")
            if (n_matches := len(matching_file_paths)) == 0:
                raise ExitCodeException(
                    f"No files found matching the regex: {src_file}",
                    EXIT_CODE_NO_FILE_MATCHES,
                )
            logger.info(
                f"{n_matches} files found. Preparing to compress with {compression}..."
            )
            compress(matching_file_paths, target_path, compression)
            logger.info(f"All files were compressed into {target_path}")
        else:
            compress([src_path], target_path, compression)
            logger.info(f"Successfully compressed {src_path} into {target_path}")

    except FileNotFoundError as e:
        logger.error(f"File {src_path} not found")
        sys.exit(EXIT_CODE_FILE_NOT_FOUND)
    except ExitCodeException as ec:
        logger.error(ec.message)
        sys.exit(ec.exit_code)
    except Exception as e:
        logger.error(
            f"An unexpected error occurred when attempting to convert files: {e}"
        )
        sys.exit(EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



import os
import re
import argparse
import sys
import shipyard_bp_utils as shipyard
from shipyard_file_manipulation import convert
from shipyard_file_manipulation.errors import (
    EXIT_CODE_UNKNOWN_ERROR,
    EXIT_CODE_FILE_NOT_FOUND,
    EXIT_CODE_NO_FILE_MATCHES,
)
from shipyard_templates import ShipyardLogger, ExitCodeException

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        default="exact_match",
        choices={"exact_match", "regex_match"},
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--destination-file-format",
        dest="destination_file_format",
        choices={"tsv", "psv", "xlsx", "parquet", "stata", "hdf5"},
        required=True,
    )
    return parser.parse_args()


def create_fallback_destination_file_name(source_file_name, destination_file_format):
    """
    If a destination_file_name is not provided, uses the source_file_name with a removal of the compression extension.
    """

    format_extensions = {
        "tsv": ".tsv",
        "psv": ".psv",
        "xlsx": ".xlsx",
        "parquet": ".parquet",
        "stata": ".dta",
        "hdf5": ".h5",
    }

    file_name = os.path.basename(source_file_name)
    file_name = (
        f"{os.path.splitext(file_name)[0]}{format_extensions[destination_file_format]}"
    )
    return file_name


def main():
    try:
        args = get_args()

        src_file = args.source_file_name
        src_dir = shipyard.files.clean_folder_name(args.source_folder_name)
        src_path = shipyard.files.combine_folder_and_file_name(
            folder_name=src_dir, file_name=src_file
        )
        match_type = args.source_file_name_match_type
        target_dir = shipyard.files.clean_folder_name(args.destination_folder_name)
        target_file = args.destination_file_name
        target_format = args.destination_file_format
        if not target_file:
            target_file = create_fallback_destination_file_name(src_file, target_format)
        target_path = shipyard.files.combine_folder_and_file_name(
            target_dir, target_file
        )

        if target_dir:
            shipyard.files.create_folder_if_dne(target_dir)

        if match_type == "regex_match":
            file_names = shipyard.files.find_all_local_file_names(src_dir)
            matching_file_names = shipyard.files.find_all_file_matches(
                file_names, re.compile(src_file)
            )
            if (n_matches := len(matching_file_names)) == 0:
                raise ExitCodeException(
                    message="No files found matching the provided regex.",
                    exit_code=EXIT_CODE_NO_FILE_MATCHES,
                )

            logger.info(f"{n_matches} files found. Preparing to convert...")
            print(f"{len(matching_file_names)} files found. Preparing to convert...")

            for index, key_name in enumerate(matching_file_names, start=1):
                target_path = shipyard.files.determine_destination_full_path(
                    destination_folder_name=target_dir,
                    destination_file_name=args.destination_file_name,
                    source_full_path=key_name,
                    file_number=index + 1,
                )
                logger.info(f"Target path is {target_path}")
                logger.info(
                    f"Converting file {index} of {len(matching_file_names)} to {target_format}"
                )
                convert(
                    src_path=key_name,
                    target_file_type=target_format,
                    target_path=target_path,
                )

        else:
            target_path = shipyard.files.determine_destination_full_path(
                destination_folder_name=target_dir,
                destination_file_name=args.destination_file_name,
                source_full_path=src_path,
            )
            convert(
                src_path=src_path,
                target_file_type=target_format,
                target_path=target_path,
            )

    except FileNotFoundError as e:
        logger.error(f"File {src_path} not found")
        sys.exit(EXIT_CODE_FILE_NOT_FOUND)
    except ExitCodeException as ec:
        logger.error(ec.message)
        sys.exit(ec.exit_code)

    except Exception as e:
        logger.error(
            f"An unexpected error occurred when attempting to convert files: {e}"
        )
        sys.exit(EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



from shipyard_templates import ExitCodeException
from typing import Union

EXIT_CODE_COMPRESSION_ERROR = 101
EXIT_CODE_DECOMPRESSION_ERROR = 102
EXIT_CODE_CONVERSION_ERROR = 103
EXIT_CODE_NO_FILE_MATCHES = 104
EXIT_CODE_FILE_NOT_FOUND = 105
EXIT_CODE_UNKNOWN_ERROR = 249


class CompressionError(ExitCodeException):
    def __init__(self, message: Union[str, Exception]):
        self.message = f"Error in attempting to compress source file: {message}"
        self.exit_code = EXIT_CODE_COMPRESSION_ERROR


class DecompressionError(ExitCodeException):
    def __init__(self, message: Union[str, Exception]):
        self.message = f"Error in attempting to decompress source file: {message}"
        self.exit_code = EXIT_CODE_DECOMPRESSION_ERROR


class ConversionError(ExitCodeException):
    def __init__(self, message: Union[str, Exception]):
        self.message = f"Error in attempting to convert source file: {message}"
        self.exit_code = EXIT_CODE_CONVERSION_ERROR



import tarfile
import pandas as pd
from zipfile import ZipFile
from typing import List
from shipyard_templates import ShipyardLogger
from shipyard_file_manipulation import errors


logger = ShipyardLogger.get_logger()


def compress(file_paths: List[str], target_path: str, compression: str):
    """
    Compress all of the matched files using the specified compression method.
    """
    try:
        if f".{compression}" in target_path:
            compressed_file_name = target_path
        else:
            compressed_file_name = f"{target_path}.{compression}"

        logger.debug(f"Files within the function are {file_paths}")
        if compression == "zip":
            with ZipFile(compressed_file_name, "w") as zip:
                for file in file_paths:
                    zip.write(file)
                    logger.info(f"Successfully compressed {file}")

        if compression == "tar.bz2":
            with tarfile.open(compressed_file_name, "w:bz2") as tar:
                for file in file_paths:
                    tar.add(file)
                    logger.info(f"Successfully compressed files")

        if compression == "tar":
            with tarfile.open(compressed_file_name, "w") as tar:
                for file in file_paths:
                    tar.add(file)
                    logger.info(f"Successfully compressed {file}")

        if compression == "tar.gz":
            with tarfile.open(compressed_file_name, "w:gz") as tar:
                for file in file_paths:
                    tar.add(file)
                    logger.info(f"Successfully compressed {file}")
    except Exception as e:
        raise errors.CompressionError(e)


def decompress(src_path: str, target_path: str, compression):
    """
    Decompress a given file, using the specified compression method.
    """

    try:
        if compression == "zip":
            with ZipFile(src_path, "r") as zip:
                zip.extractall(target_path)
            logger.info(
                f"Successfully extracted files from {src_path} to {target_path}"
            )

        if compression == "tar.bz2":
            file = tarfile.open(src_path, "r:bz2")
            file.extractall(path=target_path)
            logger.info(
                f"Successfully extracted files from {src_path} to {target_path}"
            )

        if compression == "tar":
            file = tarfile.open(src_path, "r")
            file.extractall(path=target_path)
            logger.info(
                f"Successfully extracted files from {src_path} to {target_path}"
            )

        if compression == "tar.gz":
            file = tarfile.open(src_path, "r:gz")
            file.extractall(path=target_path)
            logger.info(
                f"Successfully extracted files from {src_path} to {target_path}"
            )
    except Exception as e:
        raise errors.DecompressionError(e)


def convert(src_path: str, target_file_type: str, target_path: str):
    """
    Convert a file from one format to another.
    """
    try:
        input_df = pd.read_csv(src_path)
        extra_args = {"chunksize": 10000, "index": False}

        if target_file_type in ["tsv", "psv"]:
            if target_file_type == "tsv":
                input_df.to_csv(target_path, sep="\t", index=False)
            if target_file_type == "psv":
                input_df.to_csv(target_path, sep="|", index=False)
        if target_file_type == "xlsx":
            input_df.to_excel(target_path, engine="xlsxwriter", index=False)
        if target_file_type == "parquet":
            input_df.to_parquet(target_path, index=False)
        if target_file_type == "stata":
            input_df.to_stata(target_path)
        if target_file_type == "hdf5":
            store = pd.HDFStore(src_path)
            store.put(target_path, input_df)

        logger.info(f"Successfully converted {src_path} to {target_path}")
    except Exception as e:
        raise errors.ConversionError(e)



import argparse
import os
import sys

import shipyard_bp_utils as shipyard
from shipyard_file_manipulation import decompress
from shipyard_file_manipulation.errors import (
    EXIT_CODE_UNKNOWN_ERROR,
    EXIT_CODE_FILE_NOT_FOUND,
    EXIT_CODE_NO_FILE_MATCHES,
)
from shipyard_templates import ShipyardLogger, ExitCodeException

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--compression",
        dest="compression",
        choices={"zip", "tar", "tar.bz2", "tar.gz"},
        required=True,
    )
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--destination-file-name", dest="destination_file_name", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    return parser.parse_args()


def create_fallback_destination_file_name(source_file_name, compression):
    """
    If a destination_file_name is not provided, uses the source_file_name with a removal of the compression extension.
    """
    file_name = os.path.basename(source_file_name)
    file_name = file_name.replace(f".{compression}", "")
    return file_name


def main():
    try:
        args = get_args()
        compression = args.compression
        src_file = args.source_file_name
        src_dir = shipyard.files.clean_folder_name(args.source_folder_name)
        src_path = shipyard.files.combine_folder_and_file_name(
            folder_name=src_dir, file_name=src_file
        )
        target_dir = shipyard.files.clean_folder_name(args.destination_folder_name)
        target_file = args.destination_file_name

        if not target_file:
            target_file = create_fallback_destination_file_name(src_file, compression)

        target_path = shipyard.files.combine_folder_and_file_name(
            target_dir, target_file
        )

        if target_dir:
            shipyard.files.create_folder_if_dne(target_dir)

        logger.info("Decompressing file")
        decompress(src_path, target_path, compression)

    except FileNotFoundError as e:
        logger.error(f"File {src_path} not found")
        sys.exit(EXIT_CODE_FILE_NOT_FOUND)
    except ExitCodeException as ec:
        logger.error(ec.message)
        sys.exit(ec.exit_code)
    except Exception as e:
        logger.error(
            f"An unexpected error occurred when attempting to convert files: {e}"
        )
        sys.exit(EXIT_CODE_UNKNOWN_ERROR)

    else:
        logger.info(f"Successfully decompressed {src_path} to {target_path}")


if __name__ == "__main__":
    main()



import pandas as pd
import argparse
import os
import sys
import shipyard_bp_utils as shipyard
from shipyard_templates import ShipyardLogger
from shipyard_file_manipulation.errors import (
    EXIT_CODE_UNKNOWN_ERROR,
    EXIT_CODE_FILE_NOT_FOUND,
)


logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--source-file-name2", dest="source_file_name2", required=True)
    parser.add_argument(
        "--source-folder-name2", dest="source_folder_name2", default="", required=False
    )
    return parser.parse_args()


def main():
    try:
        args = get_args()
        src_file = args.source_file_name
        src_dir = shipyard.files.clean_folder_name(args.source_folder_name)
        src_path = shipyard.files.combine_folder_and_file_name(
            folder_name=src_dir, file_name=src_file
        )

        src_file_2 = args.source_file_name2
        src_dir_2 = shipyard.files.clean_folder_name(args.source_folder_name2)
        src_path_2 = shipyard.files.combine_folder_and_file_name(
            folder_name=src_dir_2, file_name=src_file_2
        )

        df1 = pd.read_csv(src_path)
        df2 = pd.read_csv(src_path_2)

        merged_df = df1.merge(df2, how="outer", indicator=True)
        df1_only = merged_df.loc[lambda x: x["_merge"] == "left_only"].drop(
            columns=["_merge"]
        )
        df2_only = merged_df.loc[lambda x: x["_merge"] == "right_only"].drop(
            columns=["_merge"]
        )
        df_overlap = merged_df.loc[lambda x: x["_merge"] == "both"].drop(
            columns=["_merge"]
        )

        if len(df1_only) > 0:
            df1_only_name = f"{os.path.splitext(src_file)[0]}_only.csv"
            df1_only.to_csv(df1_only_name, index=False, chunksize=10000)
            logger.info(f"Created {df1_only_name}")
        if len(df2_only) > 0:
            df2_only_name = f"{os.path.splitext(src_file_2)[0]}_only.csv"
            df2_only.to_csv(df2_only_name, index=False, chunksize=10000)
            logger.info(f"Created {df2_only_name}")
        if len(df_overlap) > 0:
            overlap_name = f"{os.path.splitext(src_file)[0]}_overlap.csv"
            df_overlap.to_csv(overlap_name, index=False, chunksize=10000)
            logger.info(f"Created {overlap_name}")
    except FileNotFoundError as e:
        logger.error(f"File not found: {e}")
        sys.exit(EXIT_CODE_FILE_NOT_FOUND)
    except Exception as e:
        logger.error(
            f"An unexpected error occurred when attempting to compare files: {e}"
        )
        sys.exit(EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()
